#include "Globals.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/keysym.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

int gamePaused = False;

int level = 0;
int incomingSpeed = 0;
int incomingRate  = 0;
int splitCheck = 0;
int stealthCheck = 0;

int numMissiles   = 0;
int missilesLeft  = 0;
long count;
int incomingCounter;

Missile missile[256];

Explosion explosions[256];
int numExplosions = 0;

int numAMMs = 0;
int totalAMMs = 0;

Missile AMM[9];

City cities[9];
int survivingCities = 0;
int score = 0;
int nextBonus = 0;

HighScore hiscores[256];
int numHighscores;

extern Pixmap backBuffer;
extern Pixmap city;
extern Pixmap rubble;
extern Pixmap landscape;
extern Display *display;
extern Window  my_window;
extern unsigned int width;
extern unsigned int height;

extern XFontStruct *fontStruct;

static int lastX = 355;
static int lastY = 200;

extern GC gc;
extern GC clearGC;
extern GC missileGC;
extern GC ammGC;
extern GC stealthGC;
extern GC explodeGC;

void Draw();
void FlipToScreen();
void GameLoop();
int  Bonus();
void ExplodeMissile(int, int, int);
void ExplodeAMM(int);
void GameOver();
void DisplayHighScores(int);
void FillInHighScore(int i);


void Wait(int milliseconds)
{
  struct timeval timeout;
    
  timeout.tv_sec  = milliseconds / 1000;
  timeout.tv_usec = (milliseconds % 1000) * 1000;

  select(FD_SETSIZE, NULL, NULL, NULL, &timeout);
}

void PrintCentred(char *string, int y)
{
  int l = strlen(string);
  int w = XTextWidth(fontStruct, string, l);

  XDrawString(display, backBuffer, missileGC, (width - w) / 2, y, string, l);
}
    
void IncrementLevel()
{
    int i;
    int totalAMMs;
    
    ++level;
    missilesLeft = 4 + (level%6+2) * 4 + (level%6+2)/4 * 32;
    incomingSpeed = level;
    if (incomingSpeed > 25) incomingSpeed = 25;
    
    incomingRate = 100 - (level%6+2) * 10;
    if (incomingRate < 4) incomingRate = 4;

    splitCheck = 20 - ((level-1)/5) * 5;
    if (splitCheck < 4) splitCheck = 4;
    stealthCheck = level > 20 ? 15 : (21 - level) * 15;

    numMissiles = 0;
    count = 0;
    incomingCounter = 0;

    totalAMMs = missilesLeft + 2*level + 4;
    totalAMMs += (totalAMMs * 2) / splitCheck;

    if (level%8 == 0) {
      incomingRate = 3;
      missilesLeft = 100;
      for (i = 0; i < 10; ++i) {
	Draw();
	if (i%2 == 0) {
	  PrintCentred("WARNING!", 192);
	}
	FlipToScreen();
	Wait(700);
      }
    }

    for(i = 0; i < 9; ++i)
    {
        if (cities[i].intact && IS_BASE(i))
        {
            cities[i].missiles = totalAMMs / 3;
        }
        else cities[i].missiles = 0;
    }
}

void DrawBase(int i)
{
  int gx, gy, fx, fy;
  double theta, gamma;
  double sinTh, cosTh, sinGm, cosGm;
  int bx, by, cx, cy, ex, ey;

  if (!IS_BASE(i)) return;

  gx = CITY_MID_X(i);
  gy = CITY_FLOOR_Y(i);
  fx = gx;
  fy = gy - 20;

  if (lastY > CITY_FLOOR_Y(i) - 20) lastY = CITY_FLOOR_Y(i) - 20;
  theta = atan2(CITY_FLOOR_Y(i) - 16 - lastY, CITY_MID_X(i) - lastX);

  gamma = (M_PI / 4.0) - theta;

  sinTh = sin(theta);
  cosTh = cos(theta);
  sinGm = sin(gamma);
  cosGm = cos(gamma);

  bx = fx - 16 * sinTh;
  by = fy + 16 * cosTh;
  cx = fx + 16 * sinTh;
  cy = fy - 16 * cosTh;

  XDrawLine(display, backBuffer, missileGC, bx, by, cx, cy);
  XDrawLine(display, backBuffer, missileGC, bx, by-1, cx, cy-1);

  XDrawLine(display, backBuffer, missileGC, bx, by, bx-8*cosGm, by+8*sinGm);
  XDrawLine(display, backBuffer, missileGC, cx, cy, cx-8*sinGm, cy-8*cosGm);

  XDrawLine(display, backBuffer, missileGC, gx, gy, fx-5*sinTh, fy+5*cosTh);
  XDrawLine(display, backBuffer, missileGC, gx, gy, fx+5*sinTh, fy-5*cosTh);

  ex = fx - 16 * cosTh;
  ey = fy - 16 * sinTh;

  XDrawLine(display, backBuffer, ammGC, fx, fy, ex, ey);
  XDrawLine(display, backBuffer, missileGC,
	    ex - 2*sinTh, ey + 2*cosTh, ex + 2*sinTh, ey - 2*cosTh);
}

void Draw()
{
    int i;
    int r;
    Pixmap src;
    char buf[32];
    
    XFillRectangle(display, backBuffer, clearGC, 0, 0, width, height - 60);
    
    XCopyPlane(display, landscape, backBuffer,
	       missileGC, 0, 0, width, 60, 0, height - 60, 1);
	       
    for(i = 0; i < 9; ++i)
    {
      if (IS_BASE(i) && cities[i].intact) {
	DrawBase(i);
      } else if (cities[i].intact) {
	XCopyPlane(display, city, backBuffer, missileGC, 0, 0, 64, 64,
		   8 + i * 80, CITY_FLOOR_Y(i)-62, 1);
      } else {
	XCopyPlane(display, rubble, backBuffer, missileGC, 0, 0, 64, 64,
		   8 + i * 80, CITY_FLOOR_Y(i)-58, 1);
      }
    }
    
    for(i = 0; i < numMissiles; ++i)
    {
      if (missile[i].xCrook > 0) {
        XDrawLine(display, backBuffer,
		  missile[i].stealth ? stealthGC : missileGC,
		  missile[i].xStart, missile[i].yStart,
		  missile[i].xCrook, missile[i].yCrook);
        XDrawLine(display, backBuffer,
		  missile[i].stealth ? stealthGC : missileGC,
		  missile[i].xCrook, missile[i].yCrook,
		  missile[i].x, missile[i].y);
      } else {
        XDrawLine(display, backBuffer,
		  missile[i].stealth ? stealthGC : missileGC,
		  missile[i].xStart, missile[i].yStart,
		  missile[i].x, missile[i].y);
      }
    }
    
    for(i = 0; i < numAMMs; ++i)
    {
      int sx = AMM[i].x - AMM[i].xInc * 12;
      int sy = AMM[i].y - AMM[i].yInc * 12;

      if (sy < 400) {
        XDrawLine(display, backBuffer, ammGC, sx, sy, AMM[i].x, AMM[i].y);
      } else {
        XDrawLine(display, backBuffer, ammGC, AMM[i].xStart, AMM[i].yStart,
		  AMM[i].x, AMM[i].y);
      }

	XDrawLine(display, backBuffer, ammGC,
		  AMM[i].xTarget - 5, AMM[i].yTarget,
		  AMM[i].xTarget + 5, AMM[i].yTarget);
	XDrawLine(display, backBuffer, ammGC,
		  AMM[i].xTarget, AMM[i].yTarget - 5,
		  AMM[i].xTarget, AMM[i].yTarget + 5);
    }

    for(i = 0; i < numExplosions; ++i)
    {
        r = explosions[i].r;
        XFillArc(display, backBuffer, explodeGC,
		 explosions[i].x - r, explosions[i].y - r,
                 r * 2, r * 2, 0, 23040);
    }
    
    sprintf(buf, "Score: %d", score);
    XDrawString(display, backBuffer, missileGC, 4, 20, buf, strlen(buf));
    sprintf(buf, "Level: %d", level);
    XDrawString(display, backBuffer, missileGC, 630, 20, buf, strlen(buf));
    
    for(i = 0; i < 9; ++i)
    {
      if (IS_BASE(i)) {
        sprintf(buf, "%d", cities[i].missiles);
        XDrawString(display, backBuffer, missileGC,
		    CITY_MID_X(i) - 8, 508, buf, strlen(buf));
      }
    }
    
    if (gamePaused) {
      PrintCentred("Paused - press P to continue . . .", 192);
    }
}

void FlipToScreen()
{
  XCopyPlane(display, backBuffer, my_window, gc, 0, 0, width, height, 0, 0, 1);
  XSync(display, False);
}

void CalculateTrajectoryFrom(int x, int y, Missile *missile,
			     double baseSpeed, int randInc, double *thetaR)
{
    double theta = atan2((missile->yTarget - y), (missile->xTarget - x));
    double mySpeed;

    if (randInc > 0) {
      mySpeed = baseSpeed + (double)(random()%randInc)/10.0;
    } else mySpeed = baseSpeed;

    missile->xInc = mySpeed * cos(theta);
    missile->yInc = mySpeed * sin(theta);    

    if (theta < 0) theta += M_PI;
    if (thetaR) *thetaR = theta;
}

void CalculateTrajectory(Missile *missile, double speed,
			 int randInc, double *theta)
{
    CalculateTrajectoryFrom(missile->xStart, missile->yStart, missile,
			    speed, randInc, theta);
}

void LaunchIncoming()
{
    int target;

    if (missilesLeft == 0) return;
    
    target = random()%9;

    missile[numMissiles].target = target;
    missile[numMissiles].xStart = random()%720;
    missile[numMissiles].yStart = 0;
    missile[numMissiles].xTarget = CITY_MID_X(target) + random()%30 - 15;
    missile[numMissiles].yTarget = CITY_ROOF_Y(target);
    missile[numMissiles].xCrook = 0;
    missile[numMissiles].yCrook = 0;
    missile[numMissiles].warhead = 48;
    CalculateTrajectory(&missile[numMissiles], 0.6, incomingSpeed, 0);
    missile[numMissiles].x = missile[numMissiles].xStart;
    missile[numMissiles].y = missile[numMissiles].yStart;
    missile[numMissiles].stealth = (random()%stealthCheck < 2);

    if (level > 1 && random()%splitCheck < 2) {
      if (level < 17 && missile[numMissiles].stealth) {
	missile[numMissiles].splitAt = -1;
      } else {
	missile[numMissiles].splitAt = random()%300;
      }	
    } else {
      missile[numMissiles].splitAt = -1;
    }
    
    ++numMissiles;
    --missilesLeft;
}    

void SplitMissile(int m)
{
  int t, nt;
  int x, y;

  if (missile[m].yTarget - missile[m].y < 150) return;

  t = missile[m].target;
  x = missile[m].x;
  y = missile[m].y;

  if (random()%10 < 2) nt = t;
  else if (t == 0) nt = 1;
  else if (t == 8) nt = 7;
  else nt = t + (random()%2)*2 - 1;

  if (nt < t) missile[m].xTarget = CITY_MID_X(t) + 10;
  else        missile[m].xTarget = CITY_MID_X(t) - 10;

  missile[m].xCrook = x;
  missile[m].yCrook = y;

  missile[numMissiles].target = nt;
  missile[numMissiles].xStart = missile[m].xStart;
  missile[numMissiles].yStart = missile[m].yStart;
  missile[numMissiles].xTarget = CITY_MID_X(nt) + random()%30 - 15;
  missile[numMissiles].yTarget = 420;
  missile[numMissiles].xCrook = x;
  missile[numMissiles].yCrook = y;
  missile[numMissiles].stealth = missile[m].stealth;
  missile[numMissiles].x = x;
  missile[numMissiles].y = y;

  missile[numMissiles].splitAt = -1;
  missile[m].splitAt = -1;

  missile[numMissiles].warhead = 24;
  missile[m].warhead = 24;

  CalculateTrajectoryFrom(x, y, &missile[m], 0.7, incomingSpeed, 0);
  CalculateTrajectoryFrom(x, y, &missile[numMissiles], 0.7, incomingSpeed, 0);

  ++numMissiles;
}    

void MoveMissiles()
{
    int i;

    for (i = 0; i < numMissiles; ++i)
    {
        missile[i].x += missile[i].xInc;
        missile[i].y += missile[i].yInc;
        
	if (missile[i].y > missile[i].yTarget)
        {
            ExplodeMissile(i, TRUE, FALSE);
            --i;
        }

	if (missile[i].splitAt >= 0 && missile[i].y > missile[i].splitAt) {
	  SplitMissile(i);
	}
    }
}


void ExplodeMissile(int m, int holocaust, int ammCaused)
{
    int i;
    
    explosions[numExplosions].x = missile[m].x;
    explosions[numExplosions].y = missile[m].y;
    explosions[numExplosions].r = 0;
    explosions[numExplosions].maxSize = missile[m].warhead;
    if (holocaust)
    {
        explosions[numExplosions].destroyedCity = missile[m].target;
    }
    else
    {
        explosions[numExplosions].destroyedCity = -1;
    }
    explosions[numExplosions].ammCaused = ammCaused;
    if (ammCaused) score += 50;
    
    ++numExplosions;
    
    for(i = m; i < numMissiles; ++i)
    {
        missile[i] = missile[i + 1];
    }
    --numMissiles;
}

void ExpandExplosions()
{
    int i, j;
    double dist;
    double dx;
    double dy;
    
    for(i = 0; i < numExplosions; ++i)
    {
        explosions[i].r += 1.5;
        if (explosions[i].r > explosions[i].maxSize)
        {
            if (explosions[i].destroyedCity != -1)
            {
                if (cities[explosions[i].destroyedCity].intact)
                {
                    cities[explosions[i].destroyedCity].intact = FALSE;
                    cities[explosions[i].destroyedCity].missiles = 0;
                    if (!IS_BASE(explosions[i].destroyedCity)) {
		      --survivingCities;
		    }
                }
            }
            
            for(j = i; j < numExplosions; ++j)
            {
                explosions[j] = explosions[j+1];
            }
            --numExplosions;
            --i;
        }
    }

    for(i = 0; i < numMissiles; ++i)
    {
        for(j = 0; j < numExplosions; ++j)
        {
            dx = missile[i].x - explosions[j].x;
            dy = missile[i].y - explosions[j].y;
            
            dist = dx * dx + dy * dy;
            if (dist < explosions[j].r * explosions[j].r)
            {
                ExplodeMissile(i, FALSE, explosions[j].ammCaused);
                --i;
                break;
            }
        }
    }
}

void LaunchAMM(int x, int y)
{
    int d2, md = -1, i;
    int target, x2, y2, da, choice = -1;

    if (numAMMs == 9 || y >= 430) return;

    /* alternative method */
    md = -1;
    choice = -1;

    for (i = 0; i < 9; ++i) {
      
      if (IS_BASE(i)) {
	
	d2 = CITY_MID_X(i) - x;
	d2 = d2 * d2;		/* after all, it has to be positive */
	
	if (cities[i].intact && cities[i].missiles > 0 &&
	    (md == -1 || d2 < md)) {
	  md = d2;
	  choice = i;
	}
      }
    }
    
    if (choice == -1) return;

    AMM[numAMMs].xStart = CITY_MID_X(choice);
    AMM[numAMMs].yStart = CITY_FLOOR_Y(choice) - 20;
    AMM[numAMMs].x = AMM[numAMMs].xStart;
    AMM[numAMMs].y = AMM[numAMMs].yStart;
    AMM[numAMMs].xTarget = x;
    AMM[numAMMs].yTarget = y;
    AMM[numAMMs].xCrook = 0;
    AMM[numAMMs].yCrook = 0;
    AMM[numAMMs].splitAt = -1;
    AMM[numAMMs].stealth = 0;
    AMM[numAMMs].warhead = 40;
    CalculateTrajectory(&AMM[numAMMs], 8, 0, &cities[choice].theta);
    ++numAMMs;
    --cities[choice].missiles;
}

void MoveAMMs()
{
    int i;

    for(i = 0; i < numAMMs; ++i)
    {
        AMM[i].x += AMM[i].xInc;
        AMM[i].y += AMM[i].yInc;
    }

    for (i = 0; i < numAMMs; ++i)
    {
       
        if (AMM[i].y < AMM[i].yTarget)
        {
            ExplodeAMM(i);
            --i;
        }
    }
}

void ExplodeAMM(int m)
{
    int i;
    
    explosions[numExplosions].x = AMM[m].x;
    explosions[numExplosions].y = AMM[m].y;
    explosions[numExplosions].r = 0;
    explosions[numExplosions].maxSize = AMM[m].warhead;
    explosions[numExplosions].destroyedCity = -1;
    explosions[numExplosions].ammCaused = TRUE;
    
    ++numExplosions;
    
    for(i = m; i < numAMMs; ++i)
    {
        AMM[i] = AMM[i + 1];
    }
    --numAMMs;

}

void StartGame()
{
    int i;

    level = 0;
    score = 0;
    nextBonus = 10000;
    
    survivingCities = 6;
    for(i = 0; i < 9; ++i)
    {
        cities[i].intact = TRUE;
	cities[i].theta = M_PI / 2;
    }

    IncrementLevel();
    Draw();

    while (survivingCities > 0)
    {
        GameLoop();
        if (Bonus()) IncrementLevel();
    }

    GameOver();
}

void GameLoop()
{
    XEvent event_received;

    XSync(display, True);
    
    while ((missilesLeft > 0 || numMissiles > 0 ||
	    numExplosions > 0) && survivingCities > 0) {

      while (XCheckMaskEvent(display, ExposureMask | KeyPressMask |
			     ButtonPressMask | PointerMotionMask,
			     &event_received)) {

	switch(event_received.type) {
	  
	case Expose:
	  FlipToScreen();
	  break;

	case MotionNotify:
	  lastX = event_received.xmotion.x;
	  lastY = event_received.xmotion.y;
	  break;
	    
	case KeyPress:
	  {
	    KeySym key = XKeycodeToKeysym(display,
					  event_received.xkey.keycode, 0);
	    if (key == XK_p) {
	      gamePaused = !gamePaused;
	      Draw();
	      FlipToScreen();
	    } else if (key == XK_q && !gamePaused) {
	      exit(0);
	    }
	  }
	  break;
	      
	case ButtonPress:
	      
	  if (event_received.xbutton.button == Button1 && !gamePaused) {
	    LaunchAMM(event_received.xbutton.x, event_received.xbutton.y);
	  }
	      
	  break;
	      
	default:
	      
	  break;
	}
      }

      if (!gamePaused) {
	
	++incomingCounter;
	if (incomingCounter > incomingRate)
	  {
	    LaunchIncoming();
	    incomingCounter = 0;
	  }
		
	MoveMissiles();
	MoveAMMs();
	ExpandExplosions();
	Draw();
	FlipToScreen();
      }
      
      {
	static int incremental = 0;
	incremental += 5;
	if (incremental >= 20) {
	  Wait(incremental);
	  incremental = 0;
	}
      }
    }
    
    XSync(display, True);
}

int Bonus()
{
    int missileBonus = 0;
    int i;
    int bonus;
    int bonusCity   = FALSE;
    int intactBonus = FALSE;
    char buf[256];
   
    if (survivingCities == 0) return False;
    
    for(i = 0; i < 9; ++i)
    {
        if (cities[i].intact && IS_BASE(i))
        {
            missileBonus += cities[i].missiles;
        }    
    }
    
    bonus = missileBonus * 25 + survivingCities * 200;

    if (score + bonus > nextBonus)
    {
        if (survivingCities == 6)
        {
            intactBonus = TRUE;
            bonusCity   = FALSE;
        }
        else
        {
            intactBonus = FALSE;
            bonusCity   = TRUE;
        }
    }

    for (i = 0; i < missileBonus + survivingCities; ++i)
    {
        Draw();

	if (i == survivingCities) Wait(1000);

	if (i < survivingCities) {
	  sprintf(buf, "City Bonus: %d", (i + 1) * 150);
	} else {
	  sprintf(buf, "City Bonus: %d", survivingCities * 150);
	}
	PrintCentred(buf, 172);

	if (i >= survivingCities) {
	  sprintf(buf, "Missile Bonus: %d", (i - survivingCities + 1) * 25);
	  PrintCentred(buf, 192);
	}

        if (bonusCity)
        {
            sprintf(buf, "Bonus Base!");
	    PrintCentred(buf, 212);
        }
        else if (intactBonus)
        {
            sprintf(buf, "All cities intact - SUPER BONUS!");
	    PrintCentred(buf, 212);
        }
        FlipToScreen();

        Wait(100);
    }

    score += bonus;
    if (intactBonus) score += 10000;
    if (bonusCity)
    {
        for (i = 0; i < 9; ++i)
        {
            if (!cities[i].intact && IS_BASE(i) && survivingCities > 0)
            {
                cities[i].intact = TRUE;
                break;
            }
        }
	if (i == 9) {
	  for (i = 0; i < 9; ++i)
	    {
	      if (!cities[i].intact && !IS_BASE(i))
		{
		  cities[i].intact = TRUE;
		  survivingCities++;
		  break;
		}
	    }
	}
    }
    nextBonus = (score / 10000 + 1) * 10000;

    Wait(1000);
    return True;
}

void GameOver()
{
    char buf[256];
    
    numMissiles = 0;
    numExplosions = 0;
    numAMMs = 0;

    Draw();

    sprintf(buf, "Game Over");
    PrintCentred(buf, 200);

    FlipToScreen();

    sleep(5);

    DisplayHighScores(UpdateHighScores());
}

int UpdateHighScores()
{
    FILE *fp;
    char *filename;
    int i, j, pos = -1;
    int added = FALSE;
    char buf[256];
   
    numHighscores = 0;
    filename = getenv("XMC_HISCORES");

    if (!filename)
    {
        filename = getenv("HOME");
        if (!filename) return;
        sprintf(buf, "%s/.xmc", filename);
        filename = buf;
    }

    if ((fp = fopen(filename, "r+")) != NULL)
    {
      for (i = 0; i < 255; ++i) {

	if (fscanf(fp, "%s %d %d", hiscores[i].name,
		   &hiscores[i].score, &hiscores[i].level) != 3) break;
	
	++numHighscores;
      }

      fclose(fp);
    }
    
    for (i = 0; i < numHighscores; ++i)
    {
        if (score > hiscores[i].score)
        {
            pos = i;
            
            for(j = numHighscores; j > pos; --j)
            {
                hiscores[j] = hiscores[j - 1];
            }

            FillInHighScore(pos);
            
            if (numHighscores < 255) ++numHighscores;
            added = TRUE;
            
            break;
        }
    }

    if (!added && numHighscores < 255)
    {
        FillInHighScore(numHighscores);
        pos = numHighscores;
        ++numHighscores;
        added = TRUE;
    }

    if (added)
    {
        if ((fp = fopen(filename, "w+")) == NULL) return;
        for(i = 0; i < numHighscores; ++i)
        {
            fprintf(fp, "%s %d %d\n", hiscores[i].name,
                    hiscores[i].score, hiscores[i].level);
        }
        fclose(fp);
    }
    
    return pos;
}

void FillInHighScore(int i)
{
    cuserid(hiscores[i].name);
    hiscores[i].score = score;
    hiscores[i].level = level;
}

void DisplayHighScores(int pos)
{
    int andOnward = FALSE;
    int i, j;
    char buf[256];
    int blink = TRUE;
    XEvent event_received;
    int count = 0;
    HighScore personal[10];
    int np = 0, ppos = -1;

    XSync(display, True);

    for (i = 0; i < numHighscores; ++i) {
      for (j = 0; j < np; ++j) {
	if (!strcmp(hiscores[i].name, personal[j].name)) break;
      }
      if (j >= np && j < 10) {
	personal[j] = hiscores[i];
	if (i == pos) ppos = j;
	++np;
      }
    }

    while (!andOnward)
    {
      while (XCheckWindowEvent(display, my_window,
			       ButtonPressMask | KeyPressMask,
			       &event_received))
	{
	  switch(event_received.type)
            {
            case ButtonPress:
	      
	      if (event_received.xbutton.button == Button1)
                {
		  andOnward = TRUE;
                }

	    case KeyPress:
	      {
		KeySym key = XKeycodeToKeysym(display,
					      event_received.xkey.keycode, 0);
		if (key == XK_q) {
		  exit(0);
		}
	      }
	      break;

	    default:
	      break;
            }
        }
        
        Draw();

	if (count < 10) {

	  sprintf(buf, "Missile Command Hall Of Fame");
	  PrintCentred(buf, 40);

	  sprintf(buf, "Click to Continue");
	  PrintCentred(buf, 430);

	  for (i = 0; i < 10 && i < numHighscores; ++i)
	    {
	      if (i != pos || blink)
		{
		  XDrawString(display, backBuffer, missileGC, 220, i * 30 + 80,
			      hiscores[i].name, strlen(hiscores[i].name));
		  sprintf(buf, "%d    Level %d", hiscores[i].score,
			  hiscores[i].level);
		  XDrawString(display, backBuffer, missileGC, 400, i * 30 + 80,
			      buf, strlen(buf));
		}
	    }
	} else {

	  sprintf(buf, "Personal Best Scores");
	  PrintCentred(buf, 40);

	  sprintf(buf, "Click to Continue");
	  PrintCentred(buf, 430);

	  for (i = 0; i < 10 && i < np; ++i)
	    {
	      if (i != ppos || blink)
		{
		  XDrawString(display, backBuffer, missileGC, 220, i * 30 + 80,
			      personal[i].name, strlen(personal[i].name));
		  sprintf(buf, "%d    Level %d", personal[i].score,
			  personal[i].level);
		  XDrawString(display, backBuffer, missileGC, 400, i * 30 + 80,
			      buf, strlen(buf));
		}
	    }
	}
        
	FlipToScreen();
	blink = !blink;
	if (++count > 19) count = 0;
	Wait(1000);
    }
}


