/**********************************************************/
/* xmc- Missile Command for X. Written because I'm bored. */
/**********************************************************/

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <sys/time.h>
#include <stdlib.h>

#include "Globals.h"

/************************/
/* Bitmap include files */
/************************/

#include "city.xbm"
#include "rubble.xbm"

/**********************/
/* X Global variables */
/**********************/

Display 	 *display;
Window 		  root_window, 
                  my_window;
XSizeHints	  size_hints;

XWindowAttributes win;
XGCValues         GCset;
GC                gc;
Font              font;
XFontStruct      *fontStruct;

unsigned long 	  black, white;
unsigned int	  width, 
                  height, 
                  root_width, 
                  root_height;
int               screen_num, 
                  border = 5;

/***********************************/
/* Pixmap stores for game graphics */
/***********************************/

Pixmap city;
Pixmap rubble;
Pixmap icon_pixmap;

Pixmap landscape;
Pixmap backBuffer;

GC missileGC;
GC ammGC;
GC clearGC;
GC explodeGC;
GC stealthGC;
char dashList[2] = {1, 1};
char stealthDashList[2] = {1, 5};


void Draw1X(Pixmap pixmap, int x, int y, int h)
{
  int i;

  if (x >= width) return;
  y -= (height - h);

  for (i = 0; i < h-y && i < 32; ++i) {
    if ((random()%((i*3)/2+3)) < 2) {
      XDrawPoint(display, pixmap, missileGC, x, y+i);
    }
  }
}

void DrawSlope(Pixmap pixmap, int sx, int l, int sy, int inc, int h)
{
  int i, y = sy;

  for (i = 0; i < l; ++i) {
    Draw1X(pixmap, sx + i, y, h);
    y += inc;
  }
}

void DrawLevel(Pixmap pixmap, int sx, int l, int y, int h)
{
  int i;

  for (i = 0; i < l; ++i) {
    Draw1X(pixmap, sx + i, y, h);
  }
}

void DrawGround(Pixmap pixmap, int wd, int ht)
{
  int i;

  XFillRectangle(display, pixmap, clearGC, 0, 0, wd, ht);

  for (i = 0; i < 9; ++i) {
    int myy = CITY_FLOOR_Y(i);

    if (IS_BASE(i) && i > 0) {

      int inc = (myy - CITY_FLOOR_Y(i-1))/8;
      DrawSlope(pixmap, i*80, 8, CITY_FLOOR_Y(i-1), inc, ht);
      DrawLevel(pixmap, i*80+8, 64, myy, ht);
      DrawSlope(pixmap, i*80+72, 8, myy, -inc, ht);

    } else {

      DrawLevel(pixmap, i*80, 80, myy, ht);
    }
  }
}


void MakePixmap(Pixmap *pixmap, char *bitmap, int wd, int ht)
{
    *pixmap = XCreatePixmapFromBitmapData
      (display, my_window, bitmap, wd, ht, white, black, 1);
    return;
}

int main(int argc, char **argv)
{
    struct timeval tp;
    struct timezone tzp;
 
    char *display_name = NULL;
    char *name = "Missile Command";

    Cursor cursor;
    XSetWindowAttributes setattr;

    gettimeofday(&tp, &tzp);
    srandom(tp.tv_usec);

    if ( (display = XOpenDisplay(display_name)) == NULL)
    {
	printf("Couldn't open display \n");
	exit(-1);
    }

    screen_num = DefaultScreen(display);

    root_width = DisplayWidth(display, screen_num);
    root_height = DisplayHeight(display, screen_num);

    width = 720;
    height = 512;

    root_window = DefaultRootWindow(display);

    black = BlackPixel(display, screen_num);
    white = WhitePixel(display, screen_num);

    fontStruct = XLoadQueryFont(display, FONT);
    font = fontStruct->fid;

    cursor = XCreateFontCursor(display, XC_tcross);
    setattr.cursor = cursor;
    setattr.backing_store = NotUseful;
    setattr.backing_planes = 0L;
    setattr.save_under = False;

    my_window = XCreateWindow
      (display, root_window, 100, 100, width, height, 
       3, CopyFromParent, CopyFromParent, CopyFromParent,
       CWCursor | CWBackingStore | CWBackingPlanes | CWSaveUnder, &setattr);

    XGetWindowAttributes(display, my_window, &win);

    backBuffer = XCreatePixmap(display, my_window, width, height, 1);
    landscape = XCreatePixmap(display, my_window, width, 60, 1);
    
    GCset.graphics_exposures = True;
    GCset.foreground = white;
    GCset.background = black;
    gc = XCreateGC(display, my_window, 
                   GCGraphicsExposures | GCForeground | GCBackground, 
                   &GCset);

    missileGC = XCreateGC(display, backBuffer, 
                          GCGraphicsExposures | GCForeground | GCBackground, 
                          &GCset);

    XSetForeground(display, missileGC, white);
    XSetBackground(display, missileGC, black);
    XSetFont(display, missileGC, font);
    
    ammGC = XCreateGC(display, backBuffer, 
                      GCGraphicsExposures | GCForeground | GCBackground, 
                      &GCset);
    XSetForeground(display, ammGC, white);
    XSetBackground(display, ammGC, black);
    XSetLineAttributes(display, ammGC, 0, LineOnOffDash, CapButt, JoinBevel);
    XSetDashes(display, ammGC, 0, dashList, 2);
    
    XSetFont(display, ammGC, font);
    
    stealthGC = XCreateGC(display, backBuffer, 
                      GCGraphicsExposures | GCForeground | GCBackground, 
                      &GCset);
    XSetForeground(display, stealthGC, white);
    XSetBackground(display, stealthGC, black);
    XSetLineAttributes(display, stealthGC, 0, LineOnOffDash, CapButt,JoinBevel);
    XSetDashes(display, stealthGC, 0, stealthDashList, 2);
    
    XSetFont(display, stealthGC, font);

    explodeGC = XCreateGC(display, backBuffer,
                          GCGraphicsExposures | GCForeground | GCBackground,
                          &GCset);
    XSetFunction(display, explodeGC, GXxor);

    GCset.graphics_exposures = False;
    GCset.foreground = BlackPixel(display, screen_num);
    GCset.background = WhitePixel(display, screen_num);

    clearGC = XCreateGC(display, backBuffer, 
                        GCGraphicsExposures | GCForeground | GCBackground, 
                        &GCset);
    XSetForeground(display, clearGC, black);
    XSetBackground(display, clearGC, white);
    
    icon_pixmap = XCreateBitmapFromData(display, my_window, city_bits,
					city_width,
					city_height);
    
    MakePixmap(&city, city_bits, city_width, city_height);
    MakePixmap(&rubble, rubble_bits, rubble_width, rubble_height);
    DrawGround(landscape, width, 60);

    size_hints.flags = 0L;
    XSetStandardProperties(display, my_window, name, name, 
			   icon_pixmap, argv, argc, &size_hints);
    
    XSelectInput(display, my_window,
		 ExposureMask | KeyPressMask | ButtonPressMask |
		 PointerMotionMask);

    XMapWindow(display, my_window);
    while(1)
        StartGame();

    XFreeGC(display, gc);
    XFreeGC(display, missileGC);
    XFreeGC(display, ammGC);
    XFreeGC(display, stealthGC);
    XFreeGC(display, clearGC);
    XFreeGC(display, explodeGC);
    XFreeFont(display, fontStruct);
}


